import component, { state } from 'kompo';
import { empty, isFunction, addClasses, isObject } from 'kompo-util';

import tableRow from './tableRow';
import tableHead from './tableHead';

export default component.construct('table', function (_ref) {
    var _this = this;

    var appendRow = _ref.appendRow,
        on = _ref.on,
        classes = _ref.classes,
        oddRowClass = _ref.oddRowClass,
        evenRowClass = _ref.evenRowClass,
        selectedClass = _ref.selectedClass,
        appendHead = _ref.appendHead,
        headRowClass = _ref.headRowClass,
        emptyNotice = _ref.emptyNotice,
        emptyClass = _ref.emptyClass;

    var head = document.createElement('thead'),
        body = document.createElement('tbody'),
        props = component.getProps(this),
        frag = document.createDocumentFragment();

    if (isFunction(on)) on(this);

    addClasses(this, classes);

    var wasEmpty = false;

    component.react(this, function (state) {
        var data = state.data;
        var offset = state.offset,
            limit = state.limit,
            minimize = state.minimize,
            rowFilter = props.rowFilter,
            columnFilter = props.columnFilter;

        if (!Array.isArray(data)) {
            if (isObject(data)) {
                data = Object.values(data);
            } else {
                return;
            }
        }

        // Empty out before append
        empty(head);
        component.unmountAll(_this);

        // When empty...
        if (data.length == 0) {
            // ... render empty notice
            var tr = document.createElement('tr'),
                td = document.createElement('td');

            td.textContent = emptyNotice;
            tr.appendChild(td);
            head.appendChild(tr);
            _this.classList.add(emptyClass);
            wasEmpty = true;
            empty(body);
            return;
        } else if (wasEmpty) {
            // Remove empty notice
            _this.classList.remove(emptyClass);
            wasEmpty = false;
        }

        var hasRowFilter = isFunction(rowFilter),
            hasColumnFilter = isFunction(columnFilter);

        appendHead(_this, head, {
            defaultClass: headRowClass,
            minimizeWhitelist: props.minimizeWhitelist,
            hasColumnFilter: hasColumnFilter,
            columnFilter: columnFilter
        });

        var dataLength = data.length;
        var os = offset && offset <= dataLength ? offset : 0;
        var l = typeof limit !== 'undefined' && os + limit < dataLength ? limit : dataLength;

        for (os; os < l; ++os) {
            if (hasRowFilter && !rowFilter(data[os])) continue;
            appendRow(_this, frag, os, {
                defaultClass: os % 2 == 0 ? evenRowClass : oddRowClass,
                selectedClass: selectedClass,
                hasColumnFilter: hasColumnFilter,
                columnFilter: columnFilter
            });
        }

        empty(body);
        body.appendChild(frag);
    });

    this.appendChild(head);
    this.appendChild(body);
}, {
    classes: [],
    oddRowClass: '',
    evenRowClass: '',
    headRowClass: '',
    minimizeWhitelist: undefined,
    emptyNotice: 'No data found',
    emptyClass: 'empty',
    appendRow: function appendRow(table, frag, key, props) {
        var tr = tableRow(props);
        component.mount(table, tr, function (s) {
            var data = table.kompo.selector ? table.kompo.selector(s).data : s.data;

            if (checkRowExistance(data, key)) return data[key];

            return {};
        });
        frag.appendChild(tr);
    },
    appendHead: function appendHead(table, head, props) {
        var tr = tableHead(props);
        component.mount(table, tr, function (s) {
            var data = table.kompo.selector ? table.kompo.selector(s).data : s.data;

            if (checkRowExistance(data, 0)) return data[0];

            return {};
        });
        head.appendChild(tr);
    }
});

function checkRowExistance(data, key) {
    return Array.isArray(data) && typeof data[key] !== 'undefined';
}

export var tableActions = {
    addLimit: function addLimit(Element, add) {
        state.dispatch(Element, function (s) {
            s.limit = s.limit + add > s.data.length ? s.data.length : s.limit + add;
        });
    },
    subLimit: function subLimit(Element, sub) {
        state.dispatch(Element, function (s) {
            s.limit = s.limit - sub <= 0 ? 0 : s.limit - sub;
        });
    },
    setLimit: function setLimit(Element, limit) {
        state.dispatch(Element, function (s) {
            if (limit <= 0) {
                s.limit = 0;
                return;
            }

            if (limit > s.data.length) {
                s.limit = s.data.length;
                return;
            }

            s.limit = limit;
        });
    },
    addOffset: function addOffset(Element, add) {
        state.dispatch(Element, function (s) {
            s.offset = s.offset + add >= s.data.length ? s.data.length : s.offset + add;
        });
    },
    subOffset: function subOffset(Element, sub) {
        state.dispatch(Element, function (s) {
            s.offset = s.offset - sub <= 0 ? 0 : s.offset - sub;
        });
    },
    setOffset: function setOffset(Element, offset) {
        state.dispatch(Element, function (s) {
            if (offset <= 0) {
                s.offset = 0;
                return;
            }

            if (offset > s.data.length) {
                s.offset = s.data.length;
                return;
            }

            s.offset = offset;
        });
    },
    minimize: function minimize(Element) {
        state.dispatch(Element, function (s) {
            s.minimize = true;
        });
    },
    maximize: function maximize(Element) {
        state.dispatch(Element, function (s) {
            s.minimize = false;
        });
    }
};